// #![no_std]
// #![no_main]

mod crypto;

use arduino_hal::hal::usart::BaudrateArduinoExt;
use arduino_hal::prelude::*;
use arduino_hal::{delay_ms, Usart};
use binascii::b32decode;
use crypto::hmac_sha1;
use std::time::{Duration, SystemTime, UNIX_EPOCH};

// use panic_halt as _;

const INTERVAL: u64 = 30;

fn process_secret<'a>(secret: &str, output: &mut [u8]) -> usize {
    let mut result = [0u8; 32];
    let mut inc = 0;
    let offset = 8 - (secret.len() % 8);

    for c in secret.chars() {
        result[inc] = c.to_ascii_uppercase() as u8;
        inc += 1;
    }
    for _ in 0..offset {
        result[inc] = '=' as u8;
        inc += 1;
    }

    let decoded = match binascii::b32decode(&result[..inc], output).ok() {
        Some(bytes) => bytes,
        None => "".as_bytes(),
    };
    decoded.len()
}

fn generate_otp_token(private_key: &str, actual_time: u64) -> u64 {
    let mut output = [0u8; 200];
    let private_key_len = process_secret(private_key, &mut output);

    let interval = actual_time / INTERVAL;
    let msg = interval.to_be_bytes();

    println!("key:\n{private_key:?}");
    println!("msg:\n{msg:?}");

    let mut hmac_digest = [0u8; 20];
    hmac_sha1(&output[..private_key_len], &msg, &mut hmac_digest);
    println!("{:?}", hmac_digest);

    let start = (hmac_digest[19] & 0xF) as usize;
    println!("Start before process: {}", hmac_digest[19]);
    println!("start: {start}");
    let bytes: [u8; 4] = hmac_digest[start..start + 4].try_into().unwrap();
    println!("bytes: {bytes:?}");

    let raw_token = u32::from_be_bytes(bytes);
    println!("raw token: {raw_token}");
    ((raw_token & 0x7F_FF_FF_FF) % 1_000_000) as u64
}

fn main() {
    // let now = SystemTime::now();
    // let elapsed = now.duration_since(UNIX_EPOCH).unwrap();
    // let seconds = elapsed.as_secs();
    // println!("seconds: {seconds}");

    // let otp_token = generate_otp_token("holaaaaamundo", seconds);

    // println!("{:06}", otp_token);
    // std::thread::sleep(Duration::from_secs(1));

    loop {
        let now = SystemTime::now();
        let elapsed = now.duration_since(UNIX_EPOCH).unwrap();
        let seconds = elapsed.as_secs();

        let otp_token = generate_otp_token("holaaaaamundo", seconds);
        println!("{:06}", otp_token);
        std::thread::sleep(Duration::from_secs(1));
    }
}

// #[arduino_hal::entry]
// fn main() -> ! {
//     let dp = arduino_hal::Peripherals::take().unwrap();
//     let pins = arduino_hal::pins!(dp);
//
//     let mut led = pins.d13.into_output();
//     let mut serial = Usart::new(
//         dp.USART0,
//         pins.d0,
//         pins.d1.into_output(),
//         BaudrateArduinoExt::into_baudrate(9600),
//     );
//
//     loop {
//         led.toggle();
//         delay_ms(1000);
//     }
// }
